; $Id: findselectedsoftcamera.pro,v 1.1 1998/01/26 17:54:59 rgr Exp $

; $RCSfile: findselectedsoftcamera.pro,v $
; $Author: rgr $
; $Date: 1998/01/26 17:54:59 $
; $Revision: 1.1 $


function FindSelectedSoftCamera, second1996

   common CrisSymbols

   common Static_FindSelectedSoftCamera, ReferenceSec, camera

   span = 86400

   if (n_elements(camera) eq 0) then begin
      ReferenceSec = 0L
      camera = bytarr(span)
   endif

   if ((ReferenceSec gt second1996) or                                $
       (second1996 ge (ReferenceSec + span))) then begin
      camera(*) = G$_Cris.UndefinedSoftCamera
      ReferenceSec = second1996

      ; Get the command tables that might contain information about
      ; the SOFT camera selected during the interval

      ncycle = (G$_Cris.NumCmdTabBytes - 1) /                         $
                                   G$_Cris.NumCmdTabBytesPerCycle + 1
      sec0 = ReferenceSec - (ncycle * G$_Cris.NumMnfsPerCycle)
      sec1 = ReferenceSec + span + (ncycle * G$_Cris.NumMnfsPerCycle)
      ct = get_group_cmd_tables_c(sec0, sec1)
      ntable = n_elements(ct) - 1

      ; Set the camera state for the times spanned by the command
      ; tables which have consistent values for the SOFT camera
      ; selected with the previous and following command tables

      sec0 = ReferenceSec
      sec1 = ReferenceSec + span - 1
      QualityMask = G$_Cris.QualityFillLevel0Bit or                   $
                    G$_Cris.QualityFillLevel1Bit

      index = where(((ct.QualityCamsel and QualityMask) eq 0B) and    $
                    (ct.Second1996MinorFrame1 ge sec0) and            $
                    (ct.Second1996MinorFrame0 le sec1), ncount)
      interval = long(ncycle * G$_Cris.NumMnfsPerCycle)
      for i=0, ncount-1 do begin
         ii = index(i)

         if (ii eq 0) then goto, AcceptPreviousTable
         if ((ct(ii-1).QualityCamsel and  QualityMask) ne 0) then     $
                                            goto, AcceptPreviousTable
         if (ct(ii-1).Second1996MinorFrame1 lt                        $
             (ct(ii).Second1996minorFrame0 - interval/2)) then        $
                                            goto, AcceptPreviousTable

         if (ct(ii-1).Camsel ne ct(ii).Camsel) then  goto, RejectTable
 
         AcceptPreviousTable:

         if (ii eq (ntable-1)) then goto, AcceptNextTable
         if ((ct(ii+1).QualityCamsel and  QualityMask) ne 0) then     $
                                                goto, AcceptNextTable
         if (ct(ii).Second1996MinorFrame1 lt                          $
             (ct(ii+1).Second1996minorFrame0 - interval/2)) then      $
                                                goto, AcceptNextTable
         if (ct(ii).Camsel ne ct(ii+1).Camsel) then goto, RejectTable

         AcceptNextTable:

         range0 = sec0 > ct(ii).Second1996MinorFrame0
         range1 = sec1 < ct(ii).Second1996MinorFrame1
         camera(range0-sec0 : range1-sec0) = ct(ii).camsel

         RejectTable:

      endfor

      ; Get the housekeeping records that that might contain 
      ; inforamtion about the SOFT camera selected during the interval

      sec0 = ReferenceSec - (2 * G$_Cris.NumMnfsPerCycle)
      sec1 = ReferenceSec + span + (2 * G$_Cris.NumMnfsPerCycle)
      hskp = get_group_hskps_c(sec0, sec1)
      nhskp = n_elements(hskp) - 1

      ; Set the camera state for the times spanned by the 
      ; housekeeping records which have consistent values for the SOFT
      ; camera selected with the previous and following housekeeping
      ; records.  The camera state can be determined can be determined
      ; by comparing the high voltage values for the two SOFT camera
      ; systems.

      sec0 = ReferenceSec
      sec1 = ReferenceSec + span - 1
      interval = G$_Cris.NumMnfsPerCycle

      index = where(                                                  $
               ((hskp.QualityMonitorMcp1 and QualityMask) eq 0B) and  $
               ((hskp.QualityMonitorMcp2 and QualityMask) eq 0B) and  $
               ((hskp.Second1996 + interval - 1) ge sec0) and         $
               (hskp.Second1996 le sec1), ncount)
      for i=0, ncount-1 do begin
         ii = index(i)

         if (ii eq 0) then goto, AcceptPreviousRecord
         if ((hskp(ii-1).QualityMonitorMcp1 and QualityMask) ne 0)    $
                                     then  goto, AcceptPreviousRecord
         if ((hskp(ii-1).QualityMonitorMcp2 and QualityMask) ne 0)    $
                                     then  goto, AcceptPreviousRecord
         if (hskp(ii-1).Second1996 lt                                 $
            (hskp(ii).Second1996 - interval/2))                       $
                                      then goto, AcceptPreviousRecord

         if (hskp(ii-1).MonitorMcp1 le hskp(ii-1).MonitorMcp2) then   $
            cam0 = 0B                                                 $
         else                                                         $
            cam0 = 1B 
         if (hskp(ii).MonitorMcp1 le hskp(ii).MonitorMcp2) then       $
            cam1 = 0B                                                 $
         else                                                         $
            cam1 = 1B
         if (cam0 ne cam1) then goto, RejectRecord
 
         AcceptPreviousRecord:

         if (ii eq (nhskp-1)) then goto, AcceptNextRecord
         if ((hskp(ii+1).QualityMonitorMcp1 and QualityMask) ne 0)    $
                                          then goto, AcceptNextRecord
         if ((hskp(ii+1).QualityMonitorMcp2 and QualityMask) ne 0)    $
                                          then goto, AcceptNextRecord
         if (hskp(ii).Second1996 lt                                   $
            (hskp(ii+1).Second1996 - interval/2))                     $
                                          then goto, AcceptNextRecord

         if (hskp(ii).MonitorMcp1 le hskp(ii).MonitorMcp2) then       $
            cam1 = 0B                                                 $
         else                                                         $
            cam1 = 1B
         if (hskp(ii+1).MonitorMcp1 le hskp(ii+1).MonitorMcp2) then   $
            cam2 = 0B                                                 $
         else                                                         $
            cam2 = 1B 
         if (cam1 ne cam2) then goto, RejectRecord

         AcceptNextRecord:

         if (ii eq (nhskp - 1)) then                                  $
            hskp(nhskp).Second1996 =                                  $
                                  hskp(nhskp-1).Second1996 + interval
         range0 = sec0 > hskp(ii).Second1996
         range1 = sec1 < hskp(ii).Second1996 + interval
         range1 = range1 < hskp(ii+1).Second1996
         if (hskp(ii).MonitorMcp1 le hskp(ii).MonitorMcp2) then       $
            cam1 = 0B                                                 $
         else                                                         $
            cam1 = 1B
         camera(range0-sec0 : range1-sec0) = cam1

         RejectRecord:

      endfor
   endif

   return, camera(second1996 - ReferenceSec)

end
