; $Id: get_group_hi_rates_se.pro,v 1.2 1998/08/03 13:36:57 rgr Exp $

; $RCSfile: get_group_hi_rates_se.pro,v $
; $Author: rgr $
; $Date: 1998/08/03 13:36:57 $
; $Revision: 1.2 $

; This function generates and returns the L1_1SisHighPriorityRate
; structures between a start and stop time.  The function uses the
; information in the L1SisHighPriorityRate structure and the command
; table to generate the structure.
;
; Arguments
;    sec0_1996 - start time in seconds of 1996-(input)
;    sec1_1996 - stop time in seconds of 1996-(input)
;
;
; Procedures and functions used
;    get_group_cmd_tables_s
;    get_group_hi_rates_s
;    scaleSisHighPriorityRate

function get_group_hi_rates_se, sec0_1996, sec1_1996

   common SisSymbols

   NCYCLE_BYTE = G$_Sis.NumCmdTabBytesPerCycle
   NMNF = G$_Sis.NumMnfsPerCycle
   NRATE = G$_Sis.NumHighPriorityRates
   NTABLE_BYTE = G$_Sis.NumCmdTabBytes

   hp = get_group_hi_rates_s(sec0_1996, sec1_1996)
   nrec = n_elements(hp) - 1
   engr = replicate({L1_1SisHighPriorityRate}, nrec + 1)

   engr.ClockMinorFrame = hp.ClockMinorFrame
   engr.Second1996 = hp.Second1996
   engr.microsecond = hp.microsecond
   engr.hp = uncompressSisValue(hp.hp)
   engr.QualityHp = hp.QualityHp

   if (nrec gt 0) then begin

      ; Get the command tables that cover the time period of the
      ; high priority rates.  If a data gap precedes the first
      ; sample of the high priority rates, then search upto
      ; 48 hours before the first high priority rate sample for
      ; an appropriate command table.  If a table cannot be found
      ; in the 48 hour time period, then use the first command
      ; table that is in the period of the high priority rates.

      second0 = hp[0].Second1996
      second1 = hp[nrec-1].Second1996 +                               $
                     NMNF * ((NTABLE_BYTE +NCYCLE_BYTE) / NCYCLE_BYTE)
      ct = get_group_cmd_tables_s(second0, second1)
      if ((ct[0].Second1996MinorFrame0 eq 0L) or                      $
         (ct[0].Second1996MinorFrame0 gt hp[0].Second1996)) then begin
         ct = get_group_cmd_tables_s(second0 - (48L*3600L), second1)
         set = where(ct.Second1996MinorFrame0 le second0, nset)
         if (nset gt 1) then                                      $
            second0 = ct[set[nset-2]].Second1996MinorFrame0 + 1L
         ct = get_group_cmd_tables_s(second0, second1)
      endif
      ncmd = n_elements(ct) - 1
      if (ncmd eq 0) then begin
         engr.ClockMinorFrame = 0L
         engr.Second1996 = 0L
         engr.microsecond = 0L
         engr.hp = 0.0
         engr.QualityHp = 0B
         return, engr[nrec]
      endif
      ct = ct[0 : ncmd-1]

      ; Store the definitions from the command table and their
      ; quality indicators into the structures.

      for i=0, NRATE - 1 do begin
         definition = ishft(long(ct.hiprcmd[2*i]), 16)
         definition = definition or                                   $
                           (long(ct.hiprcmd[2*i +1]) and '0000ffff'XL)
         engr.definition[i] = definition[0]
         set = where (definition ne definition[0], ndiff)
         for j=0, ndiff-1 do begin
            index = where (engr.ClockMinorFrame ge                    $
                              ct[set[j]].Second1996MinorFrame0, nengr)
            if (nengr gt 0) then                                      $
               engr[index].definition[i] = definition[set[j]]
         endfor
         quality = ct.QualityHiprcmd[2*i] or ct.QualityHiprcmd[2*i + 1]
         engr.QualityDefinition[i] = quality[0]
         set = where(quality ne quality[0], ndiff)
         for j=0, ndiff-1 do begin
            index = where (engr.ClockMinorFrame ge                    $
                              ct[set[j]].Second1996MinorFrame0, nengr)
            if (nengr gt 0) then                                 $
               engr[index].QualityDefinition[i] = quality[set[j]]
         endfor
      endfor

      ; Store the coincidence mask from the command table and its
      ; quality indicator into the structures

      mask = ct.coinmsk
      engr.CoinMask = mask[0]
      set = where (mask ne mask[0], ndiff)
      for i=0, ndiff-1 do begin
         index = where (engr.ClockMinorFrame ge                       $
                              ct[set[i]].Second1996MinorFrame0, nengr)
         if (nengr gt 0) then                  $
            engr[index].CoinMask = mask[set[i]]
      endfor
      quality = ct.QualityCoinmsk
      engr.QualityCoinMask = quality[0]
      set = where (quality ne quality[0], ndiff)
      for i=0, ndiff-1 do begin
         index = where (engr.ClockMinorFrame ge                       $
                              ct[set[i]].Second1996MinorFrame0, nengr)
         if (nengr gt 0) then                            $
            engr[index].QualityCoinMask = quality[set[i]]
      endfor

      ; Convert the high priority rates to seconds or counts/second
      ; by mulitpying by the proper scale factor

      for i=0, NRATE - 1 do begin
         scale = scaleSisHighPriorityRate(engr[0].definition[i])
         set = where(engr.definition[i] eq engr[0].definition[i], nset)
         engr[set].hp[i] = scale * engr[set].hp[i]
         set = where(engr.definition[i] ne engr[0].definition[i], ndiff)
         for j=0, ndiff - 1 do begin
            scale = scaleSisHighPriorityRate(engr[set[j]].definition[i])
            engr[set[j]].hp[i] = scale * engr[set[j]].hp[i]
         endfor
      endfor

   endif

   return, engr
end
