#ifndef GETGROUPSUBSETS_CE
   #define GETGROUPSUBSETS_CE

   /* $Id: getGroupSubsets_ce.c,v 1.1 1998/07/27 11:59:45 rgr Exp $ */

   /* $RCSfile: getGroupSubsets_ce.c,v $ */
   /* $Author: rgr $ */
   /* $Date: 1998/07/27 11:59:45 $ */
   /* $Revision: 1.1 $ */


   #include <stdlib.h>
   #include <string.h>

   #include "hdfi.h"
   #include "CrisConstants.h"
   
   #include "cris/L1CrisSubset.h"
   #include "cris/L1_1CrisSubset.h"

   #include "ace/AceNaN.h"
   #include "crisGetData/getGroupSubsets_c.h"

   #define NDATA  NUM_CRIS_TELEMETRY_BYTES

   struct L1_1CrisSubset *getGroupSubsets_ce(uint32 sec1996_0,
                                             uint32 sec1996_1,
                                                int autoFreeFlag)
   {
      static struct L1_1CrisSubset *allocatedMemory = NULL;

      struct L1_1CrisSubset *engr;
                     size_t nrec;
      struct L1_1CrisSubset *pointerEngr;
        struct L1CrisSubset *pointerSub;
        struct L1CrisSubset *sub;

      /* Do the automatic deallocation of memory.  When free is    */
      /* called with the address of a previously allocated memory  */
      /* block, free deallocates the memory block.  If the address */
      /* is NULL, free does nothing.                               */

      free(allocatedMemory);


      /* Find the number of structures and allocate and zero the */
      /* memory for them.                                        */

      pointerSub = getGroupSubsets_c(sec1996_0, sec1996_1, 0);
      sub = pointerSub;
      nrec = 0;
      while (sub->Second1996) {
         nrec++;
         sub++;
      }
      pointerEngr = calloc(nrec+1, sizeof(struct L1_1CrisSubset));



      /* Build an L1_1CrisSubset structure for each L1CrisSubset */
      /* structure present.                                      */

      sub = pointerSub;
      engr = pointerEngr;
      while (sub->Second1996) {
         engr->ClockMinorFrame = sub->ClockMinorFrame;
         engr->Second1996 = sub->Second1996;
         engr->microsecond = sub->microsecond;
         engr->FormatId = sub->FormatId;
         engr->MajorFrameCount = sub->MajorFrameCount;
         engr->MinorFrameCount = sub->MinorFrameCount;
         engr->quality = sub->quality;

         /* Convert the ADC's for voltage, currents, and */
         /* temperatures.                                */

         if (sub->MainBusVoltage != 0xff) 
            engr->MainBusVoltage =
                 (float32) (0.15054 * ((float64) sub->MainBusVoltage));
         else
            engr->MainBusVoltage = AceNaN();

         if (sub->CrisCurrent != 0xff) 
            engr->CrisCurrent =
                   (float32) (0.005602 * ((float64) sub->CrisCurrent));
         else
            engr->CrisCurrent = AceNaN();

         if (sub->CrisInternalTemperature1 != 0xff) 
            engr->CrisInternalTemperature1 = (float32) (-0.62745 *
                    ((float64) sub->CrisInternalTemperature1) + 100.0);
         else
            engr->CrisInternalTemperature1 = AceNaN(); 

         if (sub->CrisInternalTemperature2 != 0xff) 
            engr->CrisInternalTemperature2 = (float32) (-0.62745 *
                    ((float64) sub->CrisInternalTemperature2) + 100.0);
         else
            engr->CrisInternalTemperature2 = AceNaN(); 

         if (sub->CrisInterfaceTemperature != 0xff) 
            engr->CrisInterfaceTemperature = (float32) (-0.62745 *
                    ((float64) sub->CrisInterfaceTemperature) + 100.0);
         else
            engr->CrisInterfaceTemperature = AceNaN(); 

         /* Copy the telemetry data. */

         memcpy(engr->CrisData, sub->CrisData, NDATA);

         engr++;
         sub++;
      }
      free(pointerSub);


      /* Set the variable that controls the automatic deallocation */
      /* of the memory allocated.  If the user wants automatic     */
      /* deallocation of memory, set the variable equal to address */
      /* of the allocated memory; otherwise, set to NULL.          */

      allocatedMemory = autoFreeFlag ? pointerEngr : NULL;

      return pointerEngr;
   }

   #undef NDATA

#endif
