/* $Id: computeSisDataClassRates.c,v 1.1 1998/07/27 12:33:04 rgr Exp $ */

/* $RCSfile: computeSisDataClassRates.c,v $ */
/* $Author: rgr $ */
/* $Date: 1998/07/27 12:33:04 $ */
/* $Revision: 1.1 $ */

#include "hdfi.h"
#include "SisConstants.h"

#include "sis/L1SisLowPriorityRate.h"
#include "sis/L1_1SisLowPriorityRate.h"

#include "sis/uncompressSisValue.h"
#include "sis/sisGetData/findSisBufferType.h"

#define NBUF    NUM_SIS_EVT_BUFFERS
#define NCLASS  NUM_SIS_EVT_CLASSES
#define OFFSET  SIS_BUFFER_ID_OFFSET
#define Z1      SIS_Z1_BUFFER
#define Z2      SIS_Z2_BUFFER

void computeSisDataClassRates(struct L1SisLowPriorityRate *lp,
                            struct L1_1SisLowPriorityRate *engr)
{
   float64 clasacc;
   float64 clastot;
   float64 EfficiencyInverse[NCLASS];
   float64 factor;
       int i;
       int j;
   float64 livetim;
   float64 helivet;
   float64 hylivet;
       int type;

   livetim = (float64) (engr->livetim);
   if (livetim < 1.0e-6) livetim = 1.0e-6;
   helivet = (float64) (engr->helivet);
   if (helivet < 1.0e-6) helivet = 1.0e-6;
   hylivet = (float64) (engr->hylivet);
   if (hylivet < 1.0e-6) hylivet = 1.0e-6;
 

   for (i=0; i<NCLASS; i++) {
      clasacc = (float64) uncompressSisValue(lp->clasacc[i]);
      clastot = (float64) uncompressSisValue(lp->clastot[i]);
      if (clasacc == 0.0) clasacc = clastot;
      if (clasacc == 0.0) clasacc = 1.0;
      EfficiencyInverse[i] = clastot / clasacc;
   }

   for (i=0; i<NBUF; i++) {
      type = findSisBufferType(i + OFFSET);
      switch(type) {
         case Z1 :
             for (j=0; j<NCLASS; j++) {
                 factor = EfficiencyInverse[j] / hylivet;
                 engr->DataClassRate[i][j] =
                       (float32) (factor * lp->NumberEvents[i][j]);
                 engr->QualityDataClassRate[i][j] = 
                                           engr->QualityHylivet    |
                                           lp->QualityClasacc[j]   |
                                           lp->QualityClastot[j]   |
                                           lp->QualityNumberEvents;
              }
              break;
         case Z2 :
             for (j=0; j<NCLASS; j++) {
                 factor = EfficiencyInverse[j] / helivet;
                 engr->DataClassRate[i][j] =
                       (float32) (factor * lp->NumberEvents[i][j]);
                 engr->QualityDataClassRate[i][j] = 
                                           engr->QualityHelivet    |
                                           lp->QualityClasacc[j]   |
                                           lp->QualityClastot[j]   |
                                           lp->QualityNumberEvents;
              }
              break;
         default :
             for (j=0; j<NCLASS; j++) {
                 factor = EfficiencyInverse[j] / livetim;
                 engr->DataClassRate[i][j] =
                       (float32) (factor * lp->NumberEvents[i][j]);
                 engr->QualityDataClassRate[i][j] = 
                                           engr->QualityLivetim    |
                                           lp->QualityClasacc[j]   |
                                           lp->QualityClastot[j]   |
                                           lp->QualityNumberEvents;
              }
      }   
   }
}

#undef NBUF
#undef NCLASS
#undef OFFSET
#undef Z1
#undef Z2
