#ifndef GETGROUPHIRATES_SE
   #define GETGROUPHIRATES_SE

   /* $Id: getGroupHiRates_se.c,v 1.1 1998/08/03 13:23:24 rgr Exp $ */

   /* $RCSfile: getGroupHiRates_se.c,v $ */
   /* $Author: rgr $ */
   /* $Date: 1998/08/03 13:23:24 $ */
   /* $Revision: 1.1 $ */


   #include <stdlib.h>

   #include "hdfi.h"
   #include "SisConstants.h"
   
   #include "sis/L1SisCommandTable.h"
   #include "sis/L1SisHighPriorityRate.h"
   #include "sis/L1_1SisHighPriorityRate.h"

   #include "sisGetData/getGroupCmdTables_s.h"
   #include "sisGetData/getGroupHiRates_s.h"
   #include "sis/uncompressSisValue.h"
   #include "sis/sisGetData/scaleSisHighPriorityRate.h"

   #define NCYCLE_BYTE  NUM_SIS_CMD_TAB_BYTES_PER_CYCLE
   #define NMNF         NUM_SIS_MNFS_PER_CYCLE
   #define NRATE        NUM_SIS_HIGH_PRIORITY_RATES
   #define NTABLE_BYTE  NUM_SIS_CMD_TAB_BYTES

   struct L1_1SisHighPriorityRate *getGroupHiRates_se(
                                                  uint32 sec1996_0,
                                                  uint32 sec1996_1,
                                                     int autoFreeFlag)
   {
      static struct L1_1SisHighPriorityRate *allocatedMemory = NULL;

                              uint16 CoinMask;
            struct L1SisCommandTable *ct;
                              uint32 definition[NRATE];
      struct L1_1SisHighPriorityRate *engr;
        struct L1SisHighPriorityRate *hp;
                                 int i;
                                 int j;
                              size_t nct;
                              size_t nrec;
            struct L1SisCommandTable *pointerCt;
      struct L1_1SisHighPriorityRate *pointerEngr;
        struct L1SisHighPriorityRate *pointerHp;
                               uint8 quality[NRATE];
                               uint8 QualityCoinMask;
                             float64 scale[NRATE];
                              uint32 second;
                              uint32 second0;
                              uint32 second1;

      /* Do the automatic deallocation of memory.  When free is    */
      /* called with the address of a previously allocated memory  */
      /* block, free deallocates the memory block.  If the address */
      /* is NULL, free does nothing.                               */

      free(allocatedMemory);


      /* Find the number of structures and allocate and zero the */
      /* memory for them.                                        */

      pointerHp = getGroupHiRates_s(sec1996_0, sec1996_1, 0);
      hp = pointerHp;
      nrec = 0;
      while (hp->Second1996) {
         nrec++;
         hp++;
      }
      pointerEngr =
                calloc(nrec+1, sizeof(struct L1_1SisHighPriorityRate));
      if (!nrec) {
         free(pointerHp);
         return pointerEngr;
      }

      /* Get the command tables that cover the time period of the    */
      /* high priority rates.  If a data gap precedes the first      */
      /* sample of the high priority rates, then search upto         */
      /* 48 hours before the first high priority rate sample for     */
      /* an appropriate command table.  If a table cannot be found   */
      /* in the 48 hour time period, then use the first command      */
      /* table that is in the period of the high priority rates.     */

      hp = pointerHp;
      second0 = hp[0].Second1996;
      second1 = hp[nrec-1].Second1996 +
                    NMNF * ((NTABLE_BYTE + NCYCLE_BYTE) / NCYCLE_BYTE);
      ct = getGroupCmdTables_s(second0, second1, 0);
      if ((!ct->Second1996MinorFrame0) ||
                              (ct->Second1996MinorFrame0 > second0)) {
         free(ct);
         getGroupCmdTables_s(second0 - (48*3600), second1, 0);
      }
      pointerCt = ct;
      nct = 0;
      while (ct->Second1996MinorFrame0) {
         nct++;
         ct++;
      }
      if (!nct) {
         free(pointerHp);
         free(pointerCt);
         return pointerEngr;
      }
      ct = pointerCt;
      ct[nct].Second1996MinorFrame0 = second1 + 1;

      /* Build an L1_1SisHighPriorityRate structure for each */
      /* L1SisHighPriorityRate structure present.            */

      hp = pointerHp;
      engr = pointerEngr;
      for (i=0; i<nct; i++) {
         second = ct[i+1].Second1996MinorFrame0;
         CoinMask = ct[i].coinmsk;
         QualityCoinMask = ct[i].QualityCoinmsk;
         for (j=0; j<NRATE; j++) {
            definition[j] = (((uint32) ct[i].hiprcmd[2*j]) << 16) |
                       (((uint32) ct[i].hiprcmd[2*j +1]) & 0x0000ffff);
            quality[j] = ct[i].QualityHiprcmd[2*j] |
                                          ct[i].QualityHiprcmd[2*j +1];
            scale[j] = scaleSisHighPriorityRate(definition[j]);
         }
         while((hp->Second1996) && (hp->Second1996 < second)) {
            engr->ClockMinorFrame = hp->ClockMinorFrame;
            engr->Second1996 = hp->Second1996;
            engr->microsecond = hp->microsecond;
            engr->CoinMask = CoinMask;
            engr->QualityCoinMask = QualityCoinMask;
            for (j=0; j<NRATE; j++) {
               engr->hp[j] =
                  (float32) (scale[j] * uncompressSisValue(hp->hp[j]));
               engr->QualityHp[j] = hp->QualityHp[j];
               engr->definition[j] = definition[j];
               engr->QualityDefinition[j] = quality[j];
            }
       
            hp++;
            engr++;
         }
      }

      free(pointerHp);
      free(pointerCt);

      /* Set the variable that controls the automatic deallocation */
      /* of the memory allocated.  If the user wants automatic     */
      /* deallocation of memory, set the variable equal to address */
      /* of the allocated memory; otherwise, set to NULL.          */

      allocatedMemory = autoFreeFlag ? pointerEngr : NULL;

      return pointerEngr;
   }

   #undef NCYCLE_BYTE
   #undef NMNF
   #undef NRATE
   #undef NTABLE_BYTE

#endif
