/* $Id: dumpGen.c,v 1.1 1998/05/04 17:56:56 rgr Exp $ */

/* $RCSfile: dumpGen.c,v $ */
/* $Author: rgr $ */
/* $Date: 1998/05/04 17:56:56 $ */
/* $Revision: 1.1 $ */

/* This program is used to write a C function which will do a       */
/* formatted dump of selected fields of a list of data structures.  */
/* The fields and format are controlled by a template file which    */
/* contains the name of the dump function, a list of the types and  */
/* arguments for the dump function, the dump template, and a        */
/* dictionary of segments to be replaced in the template.           */
/* Optionally, the template file can redefine the delimiters for    */
/* marking segments to be replaced in the template, delimiters for  */
/* marking macro values to be substituted into the segment          */
/* variables, and the symbol used to mark macros in the segment     */
/* variables.                                                       */
/*                                                                  */
/* Arguments                                                        */
/*   infile = name of input template file                           */
/*   outfile = name of file to contain generated procedure          */
/*   rcs = Flag for embedding RCS commands in the generatedxdump    */
/*         function(optional)                                       */
/*                                                                  */
/* Functions used                                                   */
/*   buildSegmentDictionary                                         */
/*   fieldBreak                                                     */
/*   findDumpLine                                                   */
/*   genDumpLine                                                    */

#include <stdio.h>
#include <string.h>

#include "fieldBreak.c"
#include "findDumpLine.c"
#include "buildSegmentDictionary.c"
#include "genDumpLine.c"

int main(int argc, char *argv[])
{
   char line[1024][256];
   FILE *fd;
   int nline;
   int length;
   char segmentDelimiter[] = "<>";
   char macroDelimiter[] = "{}";
   char macroSymbol = '$';
   char fields[64][128];
   int nfields;
   int index;
   int index0;
   int index1;
   int include0;
   int include1;
   int ntemplate;
   char ProcedureName[128];
   char ProcName[128];
   int narg;
   char arg[64][128];
   int i;
   int ndict;
   char dictName[1024][128];
   char dictVariable[1024][128];
   char dictFormat[1024][128];
   char dictNameCompressed[1024][128];
   char code[512];

   /* Copy the contents of the template input file into the string */
   /* array called line.                                           */

   fd = fopen(argv[1], "r");
   nline = 0;
   while (fgets(line[nline], 256, fd)) {
      length = strlen(line[nline]);
      line[nline][length-1] = '\0';
      nline++;
   }
   fclose(fd);

   /* Get the delimiters for the segments to be replaced in the  */
   /* template.  The default segment delimiters are "<" and ">". */

   index = findDumpLine(nline, line, "!SegmentDelimiters:");
   if (index >= 0) {
      fieldBreak(line[index], fields, &nfields);
      if (nfields == 2) {
         segmentDelimiter[0] = fields[1][0];
         segmentDelimiter[1] = fields[1][1];
      }
      else {
         fprintf(stderr, "Error in segment delimiters.");
         exit(1);
      }
   }

   /* Get the delimiters for the macro values to be substituted into */
   /* the segment variables.  The default macro value delimiters are */
   /* "{" and "}".                                                   */

   index = findDumpLine(nline, line, "!MacroDelimiters:");
   if (index >= 0) {
      fieldBreak(line[index], fields, &nfields);
      if (nfields == 2) {
         macroDelimiter[0] = fields[1][0];
         macroDelimiter[1] = fields[1][1];
      }
      else {
         fprintf(stderr, "Error in macro delimiters.");
         exit(1);
      }
   }

   /* Get the symbol for marking macros in the segemnt variables.  */
   /* The  default macro symbol is "$".                            */

   index = findDumpLine(nline, line, "!MacroSymbol:");
   if (index >= 0) {
      fieldBreak(line[index], fields, &nfields);
      if (nfields == 2) {
         macroSymbol = fields[1][0];
      }
      else {
         fprintf(stderr, "Error in macro symbol.");
         exit(1);
      }
   }

   /* Get the name of the generated dump procedure from the */
   /* information that was in the input template file.      */

   index = findDumpLine(nline, line, "!Procedure:");
   if (index >= 0) {
      fieldBreak(line[index], fields, &nfields);
      if (nfields >= 2) {
         strcpy(ProcedureName, fields[1]);
      }
      else {
         fprintf(stderr, "Cannot locate procedure name.");
         exit(1);
      }
   }
   else {
      fprintf(stderr,  "Cannot locate procedure name.");
      exit(1);
   }

   /* Get the  arguments which will be used in the generated dump */
   /* procedure.                                                  */

   index = findDumpLine(nline, line, "!ArgumentList:");
   if (index >= 0) {
      fieldBreak(line[index], fields, &nfields);
      if (nfields >= 2) {
         for (i=1; i<nfields; i++)
            strcpy(arg[i-1], fields[i]);
         narg = nfields - 1;
      }
      else {
         fprintf(stderr, "Cannot locate argument list.");
         exit(1);
      }
   }
   else {
      fprintf(stderr, "Cannot locate argument list.");
      exit(1);
   }

   /* Find the beginning and ending of the dictionary of the        */
   /* segments to be replaced in the template and build the segment */
   /* dictionary.                                                   */

   index0 = findDumpLine(nline, line, "!BeginDictionary") + 1;
   index1 = findDumpLine(nline, line, "!EndDictionary") - 1;
   if ((index0 <= 0) || (index0 > index1)) {
      fprintf(stderr, "Cannot find dictionary.");
      exit(1);
   }
   ndict = (index1 - index0) + 1;
   buildSegmentDictionary(index0, index1, line, segmentDelimiter,
                          macroDelimiter, dictName, dictVariable,
                          dictFormat, dictNameCompressed);

   /* Find the beginning and ending of the dump template. */

   index0 = findDumpLine(nline, line, "!BeginTemplate") + 1;
   index1 = findDumpLine(nline, line, "!EndTemplate") - 1;
   if ((index0 <= 0) || (index0 > index1)) {
      fprintf(stderr, "Cannot find template.");
      exit(1);
   }

   /* Find the beginning and ending of the include section. */

   include0 = findDumpLine(nline, line, "!BeginInclude") + 1;
   include1 = findDumpLine(nline, line, "!EndInclude") - 1;

   /* Open the output file which will contain the generated dump */
   /* procedure.                                                 */

   fd = fopen(argv[2], "w");

   /* Generate the #ifndef and #define directives for the function. */

      strcpy(ProcName, ProcedureName);
      length = strlen(ProcName);
      for (i=0; i<length; i++) {
         if (('a' <= ProcName[i]) && (ProcName[i] <= 'z'))
            ProcName[i] -= 0x20;
      }
      fprintf(fd, "#ifndef %s_C\n", ProcName);
      fprintf(fd, "   #define %s_C\n", ProcName);
      fprintf(fd, "\n");

   /* Generate the RCS commands for the dump procedure. */

   if (argc > 3) {
      fprintf(fd, "   /* $Id: dumpGen.c,v 1.1 1998/05/04 17:56:56 rgr Exp $ */\n");
      fprintf(fd, "\n");
      fprintf(fd, "   /* $RCSfile: dumpGen.c,v $ */\n");
      fprintf(fd, "   /* $Author: rgr $ */\n");
      fprintf(fd, "   /* $Date: 1998/05/04 17:56:56 $ */\n");
      fprintf(fd, "   /* $Revision: 1.1 $ */\n");
      fprintf(fd, "\n");
      fprintf(fd, "\n");
   };

   /* Generate the include statements. */

   fprintf(fd, "   #include <memory.h>\n");
   fprintf(fd, "   #include <stdio.h>\n");
   if ((include0 >=0) && (include0 <= include1)) {
      for (i=include0; i<=include1; i++) 
         fprintf(fd, "   %s\n", line[i]);
   }
   fprintf(fd, "\n");

   /* Generate the function line for the dump procedure. */

   sprintf(code, "   void %s(", ProcedureName);
   for (i=0; i<narg; i++) {
      if (i) strcat(code, " ");
      strcat(code, arg[i]);
   }
   strcat(code, ")");
   fprintf(fd, "%s\n", code);
   fprintf(fd, "   {\n");

   /* Generate the code for the local variables. */

   fprintf(fd, "      char line[256];\n");
   fprintf(fd, "      char segmentValue[256];\n");

   /* Embedded the dump template into the dump procedure. */

   fprintf(fd, "      char *template[] = {\"%s\"", line[index0]);
   for (i=index0+1; i<=index1; i++)
       fprintf(fd, ",\n                          \"%s\"", line[i]);
   fprintf(fd, "};\n");
   fprintf(fd, "\n");

   /* Generate the code to fill in each line of the dump template. */

   ntemplate = 0;
   for (i=index0; i<=index1; i++) { 
      genDumpLine(fd, ntemplate, line[i], segmentDelimiter, 
                  macroDelimiter, macroSymbol, ndict, dictName,
                  dictVariable, dictFormat, dictNameCompressed);
      ntemplate++;
   }

   /* Write the closing "}" for the function, write the "#endif" and */
   /* close the file the generated function.                         */

   fprintf(fd, "   }\n");
   fprintf(fd, "\n");
   fprintf(fd, "#endif\n");

   fclose(fd);

   exit(0);
}
