#ifndef GEN_DUMP_LINE_C
   #define GEN_DUMP_LINE_C

   /* $Id: genDumpLine.c,v 1.1 1998/05/04 17:58:47 rgr Exp $ */

   /* $RCSfile: genDumpLine.c,v $ */
   /* $Author: rgr $ */
   /* $Date: 1998/05/04 17:58:47 $ */
   /* $Revision: 1.1 $ */

   /* This function generates the C code to write one line of the    */
   /* dump.  The basic procedure that is used is                     */
   /*                                                                */
   /*    1. generate code to store the template for the line in the  */
   /*       variable line,                                           */
   /*    2. for each segment located in the template,                */
   /*       a. locate the appropriate dictionary entry,              */
   /*       b. generate code to put blanks into variable line where  */
   /*          the segment is acting as a placeholder,               */
   /*       c. generate the code that stores the replacement value   */
   /*          of the segment in the variable segmentVariable,       */
   /*       d. generate the code that stores segmentVariable in the  */
   /*          variable line,                                        */
   /*       e. generate code to print the variable line.             */
   /*                                                                */
   /*  Arguments                                                     */
   /*     fd = file descriptor of the file to contain the dump       */
   /*          function-(input)                                      */
   /*     index = index of the template line being expanded-(input)  */
   /*     template = template of the line for which code is being    */
   /*                generated-(input)                               */
   /*     segmentDelimiter = delimiters for segments embedded in the */
   /*                         template-(input)                       */
   /*     macroDelimiter = delimiters for macro values embedded in   */
   /*                      the segments-(input)                      */
   /*     macroSymbol = symbol for macros in dictionary              */
   /*                   variables-(input)                            */
   /*     ndict = number of dictionary entries-(input)               */
   /*     dictName = full dictionary names of segments-(input)       */
   /*     dictVariables = dictionary variables for segments-(input)  */
   /*     dictFormat = dictionary formats for segments-(input)       */
   /*     dictNameCompressed = compressed dictionary names of        */
   /*                          segments-(input)                      */
   /*                                                                */
   /*                                                                */

   #include <stdio.h>
   #include <string.h>

   #include "compressDumpSegment.c"
   #include "expandSegmentVariables.c"


   void genDumpLine(FILE *fd,
                    int index,
                    char *template,
                    char *segmentDelimiter,
                    char *macroDelimiter,
                    char macroSymbol,
                    int ndict,
                    char dictName[][128],
                    char dictVariable[][128],
                    char dictFormat[][128],
                    char dictNameCompressed[][128])
   {
       int i;
       int lengthSeg;
      char line[256];
       int lookup = 0;
      char *mark0;
      char *mark1;
       int nlookup;
       int position;
      char seg[128];
      char segment[128];
      char variable[256];

      fprintf(fd,"      strcpy(line, template[%d]);\n", index);
      strcpy(line, template);
      mark0 = strchr(line, segmentDelimiter[0]);
      while (mark0) {
         mark1 = strchr(mark0, segmentDelimiter[1]);
         *mark1 = '\0';
         strcpy(segment, mark0+1);
         compressDumpSegment(seg, mark0+1, macroDelimiter);
         nlookup = 0;
         for (i=0; i<ndict; i++) {
            if (!strcmp(seg, dictNameCompressed[i])) {
               lookup = i;
               nlookup++;
            }
         }
         if (nlookup != 1) {
            fprintf(stderr,
                    "%s - Cannot locate unique dictionary name.\n",
                    seg);
            exit(1);
         }
         lengthSeg = strlen(mark0) + 1;
         *mark0 = '\0';
         position = strlen(line);
         fprintf(fd, "         memset(line + %d, (int) ' ', %d);\n",
                                                 position, lengthSeg);
         expandSegmentVariables(variable,
                                segment,
                                dictName[lookup],
                                dictVariable[lookup],
                                macroDelimiter,
                                macroSymbol);
         fprintf(fd, "         sprintf(segmentValue, \"%s\", %s);\n",
                                         dictFormat[lookup], variable);
         fprintf(fd, "         memcpy(line + %d, segmentValue, %s);\n",
                                      position,"strlen(segmentValue)");
         *mark0 = ' ';

         *mark1 = ' ';
         mark0 = strchr(mark1, segmentDelimiter[0]);
      }

      fprintf(fd, "         printf(\"%s\\n\", line);\n", "%s");
   }

#endif
